@@ -479,11 +479,13 @@ gdm_product_slave_create_server (GdmProductSlave *slave)
 {
         char    *display_name;
         char    *auth_file;
+        char    *seat_id;
         gboolean display_is_local;
 
         g_object_get (slave,
                       "display-is-local", &display_is_local,
                       "display-name", &display_name,
+                      "display-seat-id", &seat_id,
                       "display-x11-authority-file", &auth_file,
                       NULL);
 
@@ -492,7 +494,7 @@ gdm_product_slave_create_server (GdmProductSlave *slave)
         if (display_is_local) {
                 gboolean res;
 
-                slave->priv->server = gdm_server_new (display_name, auth_file);
+                slave->priv->server = gdm_server_new (display_name, seat_id, auth_file);
                 g_signal_connect (slave->priv->server,
                                   "exited",
                                   G_CALLBACK (on_server_exited),
@@ -998,6 +1000,7 @@ create_new_session (GdmProductSlave *slave)
         char          *display_name;
         char          *display_hostname;
         char          *display_device;
+        char          *display_seat_id;
         char          *display_x11_authority_file;
 
         g_debug ("GdmProductSlave: Creating new session");
@@ -1008,6 +1011,7 @@ create_new_session (GdmProductSlave *slave)
                       "display-hostname", &display_hostname,
                       "display-is-local", &display_is_local,
                       "display-x11-authority-file", &display_x11_authority_file,
+                      "display-seat-id", &display_seat_id,
                       NULL);
 
         /* FIXME: we don't yet have a display device! */
@@ -1017,6 +1021,7 @@ create_new_session (GdmProductSlave *slave)
                                                        display_name,
                                                        display_hostname,
                                                        display_device,
+                                                       display_seat_id,
                                                        display_x11_authority_file,
                                                        display_is_local);
         g_free (display_id);
