@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
 
@@ -27,7 +27,6 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <signal.h>
 #include <fcntl.h>
 #include <signal.h>
 #include <locale.h>
@@ -36,6 +35,11 @@
 #include <glib/gi18n.h>
 #include <glib-object.h>
 
+#define DBUS_API_SUBJECT_TO_CHANGE
+#include <dbus/dbus-glib.h>
+#include <dbus/dbus-glib-lowlevel.h>
+
+#include "gdm-signal-handler.h"
 #include "gdm-common.h"
 #include "gdm-log.h"
 #include "gdm-session-worker.h"
@@ -44,57 +48,90 @@
 #include "gdm-settings-direct.h"
 #include "gdm-settings-keys.h"
 
+#define SERVER_DBUS_PATH      "/org/gnome/DisplayManager/SessionServer"
+#define SERVER_DBUS_INTERFACE "org.gnome.DisplayManager.SessionServer"
+
 static GdmSettings *settings = NULL;
 
 static gboolean
-on_sigusr1_cb (gpointer user_data)
+signal_cb (int      signo,
+           gpointer data)
 {
-        g_debug ("Got USR1 signal");
+        int ret;
 
-        gdm_log_toggle_debug ();
+        g_debug ("Got callback for signal %d", signo);
 
-        return TRUE;
+        ret = TRUE;
+
+        switch (signo) {
+        case SIGSEGV:
+        case SIGBUS:
+        case SIGILL:
+        case SIGABRT:
+                g_debug ("Caught signal %d.", signo);
+
+                ret = FALSE;
+                break;
+
+        case SIGFPE:
+        case SIGPIPE:
+                /* let the fatal signals interrupt us */
+                g_debug ("Caught signal %d, shutting down abnormally.", signo);
+                ret = FALSE;
+
+                break;
+
+        case SIGINT:
+        case SIGTERM:
+                /* let the fatal signals interrupt us */
+                g_debug ("Caught signal %d, shutting down normally.", signo);
+                ret = FALSE;
+                break;
+
+        case SIGHUP:
+                g_debug ("Got HUP signal");
+                /* FIXME:
+                 * Reread config stuff like system config files, VPN service files, etc
+                 */
+                ret = TRUE;
+
+                break;
+
+        case SIGUSR1:
+                g_debug ("Got USR1 signal");
+                /* FIXME:
+                 * Play with log levels or something
+                 */
+                ret = TRUE;
+
+                gdm_log_toggle_debug ();
+
+                break;
+
+        default:
+                g_debug ("Caught unhandled signal %d", signo);
+                ret = TRUE;
+
+                break;
+        }
+
+        return ret;
 }
 
 static gboolean
 is_debug_set (void)
 {
-        gboolean debug;
+        gboolean debug = FALSE;
+
+        /* enable debugging for unstable builds */
+        if (gdm_is_version_unstable ()) {
+                return TRUE;
+        }
+
         gdm_settings_direct_get_boolean (GDM_KEY_DEBUG, &debug);
         return debug;
 }
 
-static gboolean
-on_shutdown_signal_cb (gpointer user_data)
-{
-        GMainLoop *mainloop = user_data;
-
-        g_main_loop_quit (mainloop);
-
-        return FALSE;
-}
-
-static void
-on_state_changed (GdmSessionWorker *worker,
-                  GParamSpec       *pspec,
-                  GMainLoop        *main_loop)
-{
-        GdmSessionWorkerState state;
-
-        g_object_get (G_OBJECT (worker), "state", &state, NULL);
-
-        if (state != GDM_SESSION_WORKER_STATE_SESSION_STARTED)
-                return;
-
-        g_unix_signal_add (SIGTERM, on_shutdown_signal_cb, main_loop);
-}
-
-static void
-on_sigterm_cb (int signal_number)
-{
-        _exit (EXIT_SUCCESS);
-}
-
 int
 main (int    argc,
       char **argv)
@@ -102,18 +139,18 @@ main (int    argc,
         GMainLoop        *main_loop;
         GOptionContext   *context;
         GdmSessionWorker *worker;
+        GdmSignalHandler *signal_handler;
         const char       *address;
-        gboolean          is_for_reauth;
         static GOptionEntry entries []   = {
                 { NULL }
         };
 
-        signal (SIGTERM, on_sigterm_cb);
-
         bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
         textdomain (GETTEXT_PACKAGE);
         setlocale (LC_ALL, "");
 
+        g_type_init ();
+
         /* Translators: worker is a helper process that does the work
            of starting up a session */
         context = g_option_context_new (_("GNOME Display Manager Session Worker"));
@@ -127,12 +164,12 @@ main (int    argc,
         settings = gdm_settings_new ();
         if (settings == NULL) {
                 g_warning ("Unable to initialize settings");
-                exit (EXIT_FAILURE);
+                exit (1);
         }
 
         if (! gdm_settings_direct_init (settings, DATADIR "/gdm/gdm.schemas", "/")) {
                 g_warning ("Unable to initialize settings");
-                exit (EXIT_FAILURE);
+                exit (1);
         }
 
         gdm_log_set_debug (is_debug_set ());
@@ -140,33 +177,39 @@ main (int    argc,
         address = g_getenv ("GDM_SESSION_DBUS_ADDRESS");
         if (address == NULL) {
                 g_warning ("GDM_SESSION_DBUS_ADDRESS not set");
-                exit (EXIT_FAILURE);
+                exit (1);
         }
 
-        is_for_reauth = g_getenv ("GDM_SESSION_FOR_REAUTH") != NULL;
-
-        worker = gdm_session_worker_new (address, is_for_reauth);
+        worker = gdm_session_worker_new (address);
 
         main_loop = g_main_loop_new (NULL, FALSE);
 
-        g_signal_connect (G_OBJECT (worker),
-                          "notify::state",
-                          G_CALLBACK (on_state_changed),
-                          main_loop);
-
-        g_unix_signal_add (SIGUSR1, on_sigusr1_cb, NULL);
+        signal_handler = gdm_signal_handler_new ();
+        gdm_signal_handler_set_fatal_func (signal_handler,
+                                           (GDestroyNotify)g_main_loop_quit,
+                                           main_loop);
+        gdm_signal_handler_add (signal_handler, SIGINT, signal_cb, NULL);
+        gdm_signal_handler_add (signal_handler, SIGILL, signal_cb, NULL);
+        gdm_signal_handler_add (signal_handler, SIGBUS, signal_cb, NULL);
+        gdm_signal_handler_add (signal_handler, SIGFPE, signal_cb, NULL);
+        gdm_signal_handler_add (signal_handler, SIGHUP, signal_cb, NULL);
+        gdm_signal_handler_add (signal_handler, SIGSEGV, signal_cb, NULL);
+        gdm_signal_handler_add (signal_handler, SIGABRT, signal_cb, NULL);
+        gdm_signal_handler_add (signal_handler, SIGUSR1, signal_cb, NULL);
 
         g_main_loop_run (main_loop);
 
         if (worker != NULL) {
-                g_signal_handlers_disconnect_by_func (worker,
-                                                      G_CALLBACK (on_state_changed),
-                                                      main_loop);
                 g_object_unref (worker);
         }
 
+        if (signal_handler != NULL) {
+                g_object_unref (signal_handler);
+        }
+
         g_main_loop_unref (main_loop);
 
+
         g_debug ("Worker finished");
 
         return 0;
